@@ -1,8 +1,8 @@
 /* BEGIN_COMMON_COPYRIGHT_HEADER
  * (c)LGPL2+
  *
- * LXDE-Qt - a lightweight, Qt based, desktop toolset
- * http://razor-qt.org
+ * LXQt - a lightweight, Qt based, desktop toolset
+ * https://lxqt.org
  *
  * Copyright: 2010-2013 Razor team
  * Authors:
@@ -31,7 +31,6 @@
 
 #include "providers.h"
 #include "yamlparser.h"
-#include <XdgIcon>
 #include <XdgDesktopFile>
 #include <XdgMenu>
 #include <XmlHelper>
@@ -44,6 +43,7 @@
 #include <QDir>
 #include <QApplication>
 #include <QAction>
+#include <LXQt/Globals>
 #include <LXQt/PowerManager>
 #include <LXQt/ScreenSaver>
 #include "providers.h"
@@ -63,7 +63,7 @@ static QString expandCommand(const QString &command, QStringList *arguments=0)
     wordexp_t words;
 
     if (wordexp(command.toLocal8Bit().data(), &words, 0) != 0)
-        return "";
+        return QString();
 
     char **w;
     w = words.we_wordv;
@@ -86,7 +86,7 @@ static QString expandCommand(const QString &command, QStringList *arguments=0)
 static QString which(const QString &progName)
 {
     if (progName.isEmpty())
-        return "";
+        return QString();
 
     if (progName.startsWith(QDir::separator()))
     {
@@ -95,16 +95,16 @@ static QString which(const QString &progName)
             return fileInfo.absoluteFilePath();
     }
 
-    QStringList dirs = QString(getenv("PATH")).split(":");
+    const QStringList dirs = QFile::decodeName(qgetenv("PATH")).split(QL1C(':'));
 
-    foreach (QString dir, dirs)
+    for (const QString &dir : dirs)
     {
         QFileInfo fileInfo(QDir(dir), progName);
         if (fileInfo.isExecutable() && fileInfo.isFile())
             return fileInfo.absoluteFilePath();
     }
 
-    return "";
+    return QString();
 }
 
 
@@ -125,8 +125,8 @@ static bool startProcess(QString command)
         //fallback for executable script with no #!
         //trying as in system(2)
         args.prepend(program);
-        args.prepend(QStringLiteral("-c"));
-        return QProcess::startDetached(QStringLiteral("/bin/sh"), args);
+        args.prepend(QSL("-c"));
+        return QProcess::startDetached(QSL("/bin/sh"), args);
     }
 
 }
@@ -174,13 +174,13 @@ CommandProvider::~CommandProvider()
 AppLinkItem::AppLinkItem(const QDomElement &element):
         CommandProviderItem()
 {
-    mIconName = element.attribute("icon");
-    mTitle = element.attribute("title");
-    mComment = element.attribute("genericName");
-    mToolTip = element.attribute("comment");
-    mCommand = element.attribute("exec");
-    mProgram = QFileInfo(element.attribute("exec")).baseName().section(" ", 0, 0);
-    mDesktopFile = element.attribute("desktopFile");
+    mIconName = element.attribute(QLatin1String("icon"));
+    mTitle = element.attribute(QLatin1String("title"));
+    mComment = element.attribute(QSL("genericName"));
+    mToolTip = element.attribute(QLatin1String("comment"));
+    mCommand = element.attribute(QL1S("exec"));
+    mProgram = QFileInfo(element.attribute(QL1S("exec"))).baseName().section(QL1C(' '), 0, 0);
+    mDesktopFile = element.attribute(QSL("desktopFile"));
     initExec();
     QMetaObject::invokeMethod(this, "updateIcon", Qt::QueuedConnection);
 }
@@ -192,10 +192,10 @@ AppLinkItem::AppLinkItem(MenuCacheApp* app):
     MenuCacheItem* item = MENU_CACHE_ITEM(app);
     mIconName = QString::fromUtf8(menu_cache_item_get_icon(item));
     mTitle = QString::fromUtf8(menu_cache_item_get_name(item));
-    mComment = QString::fromUtf8(menu_cache_item_get_comment(item));
-    mToolTip = mComment;
-    mCommand = menu_cache_app_get_exec(app);
-    mProgram = QFileInfo(mCommand).baseName().section(" ", 0, 0);
+    mComment = QString::fromUtf8(menu_cache_app_get_generic_name(app));
+    mToolTip = QString::fromUtf8(menu_cache_item_get_comment(item));
+    mCommand = QString::fromUtf8(menu_cache_app_get_exec(app));
+    mProgram = QFileInfo(mCommand).baseName().section(QL1C(' '), 0, 0);
     char* path = menu_cache_item_get_file_path(MENU_CACHE_ITEM(app));
     mDesktopFile = QString::fromLocal8Bit(path);
     g_free(path);
@@ -214,7 +214,7 @@ void AppLinkItem::updateIcon()
 //    qDebug() << hex << this;
 //    qDebug() << Q_FUNC_INFO;
     if (mIcon.isNull())
-        mIcon = XdgIcon::fromTheme(mIconName);
+        mIcon = QIcon::fromTheme(mIconName);
 //    qDebug() << "*****************************************";
 }
 
@@ -271,8 +271,10 @@ bool AppLinkItem::compare(const QRegExp &regExp) const
     if (regExp.isEmpty())
         return false;
 
-    return mProgram.contains(regExp) ||
-           mTitle.contains(regExp) ;
+    return mProgram.contains(regExp)
+        || mTitle.contains(regExp)
+        || mComment.contains(regExp)
+        || mToolTip.contains(regExp);
 }
 
 
@@ -281,11 +283,11 @@ bool AppLinkItem::compare(const QRegExp &regExp) const
  ************************************************/
 void AppLinkItem::initExec()
 {
-    static const QRegExp split_re{QStringLiteral("\\s")};
+    static const QRegExp split_re{QSL("\\s")};
     XdgDesktopFile desktop;
     if (desktop.load(mDesktopFile))
     {
-        QStringList cmd = desktop.value(QStringLiteral("Exec")).toString().split(split_re);
+        QStringList cmd = desktop.value(QSL("Exec")).toString().split(split_re);
         if (0 < cmd.size())
             mExec = which(expandCommand(cmd[0]));
     }
@@ -300,14 +302,14 @@ AppLinkProvider::AppLinkProvider():
 {
 #ifdef HAVE_MENU_CACHE
     menu_cache_init(0);
-    mMenuCache = menu_cache_lookup(XdgMenu::getMenuFileName().toLocal8Bit());
+    mMenuCache = menu_cache_lookup(XdgMenu::getMenuFileName().toLocal8Bit().constData());
     if(mMenuCache)
         mMenuCacheNotify = menu_cache_add_reload_notify(mMenuCache, (MenuCacheReloadNotify)menuCacheReloadNotify, this);
     else
         mMenuCacheNotify = 0;
 #else
     mXdgMenu = new XdgMenu();
-    mXdgMenu->setEnvironments(QStringList() << "X-LXQT" << "LXQt");
+    mXdgMenu->setEnvironments(QStringList() << QSL("X-LXQT") << QSL("LXQt"));
     connect(mXdgMenu, SIGNAL(changed()), this, SLOT(update()));
     mXdgMenu->read(XdgMenu::getMenuFileName());
     update();
@@ -347,17 +349,17 @@ void AppLinkProvider::menuCacheReloadNotify(MenuCache* cache, gpointer user_data
 
  void doUpdate(const QDomElement &xml, QHash<QString, AppLinkItem*> &items)
 {
-    DomElementIterator it(xml, "");
+    DomElementIterator it(xml, QString());
     while (it.hasNext())
     {
         QDomElement e = it.next();
 
         // Build submenu ........................
-        if (e.tagName() == "Menu")
+        if (e.tagName() == QL1S("Menu"))
             doUpdate(e, items);
 
         //Build application link ................
-        else if (e.tagName() == "AppLink")
+        else if (e.tagName() == QL1S("AppLink"))
         {
             AppLinkItem *item = new AppLinkItem(e);
             delete items[item->command()]; // delete previous item;
@@ -438,7 +440,7 @@ void AppLinkProvider::update()
 HistoryItem::HistoryItem(const QString &command):
         CommandProviderItem()
 {
-    mIcon = XdgIcon::defaultApplicationIcon();
+    mIcon = QIcon::fromTheme(QLatin1String("application-x-executable"));
     mTitle = command;
     mComment = QObject::tr("History");
     mCommand = command;
@@ -478,12 +480,12 @@ unsigned int HistoryItem::rank(const QString &pattern) const
 HistoryProvider::HistoryProvider():
         CommandProvider()
 {
-    QString fileName = (XdgDirs::cacheHome() + "/lxqt-runner.history");
+    QString fileName = (XdgDirs::cacheHome() + QSL("/lxqt-runner.history"));
     mHistoryFile = new QSettings(fileName, QSettings::IniFormat);
-    mHistoryFile->beginGroup("commands");
+    mHistoryFile->beginGroup(QSL("commands"));
     for (uint i=0; i<MAX_HISTORY; ++i)
     {
-        QString key = QString("%1").arg(i, 3, 10, QChar('0'));
+        QString key = QString::fromLatin1("%1").arg(i, 3, 10, QL1C('0'));
         if (mHistoryFile->contains(key))
         {
             HistoryItem *item = new HistoryItem(mHistoryFile->value(key).toString());
@@ -526,7 +528,7 @@ void HistoryProvider::AddCommand(const QString &command)
     mHistoryFile->clear();
     for (int i=0; i<qMin(length(), MAX_HISTORY); ++i)
     {
-        QString key = QString("%1").arg(i, 3, 10, QChar('0'));
+        QString key = QString::fromLatin1("%1").arg(i, 3, 10, QL1C('0'));
         mHistoryFile->setValue(key, static_cast<HistoryItem*>(at(i))->command());
     }
 }
@@ -544,7 +546,7 @@ CustomCommandItem::CustomCommandItem(CustomCommandProvider *provider):
     CommandProviderItem(),
     mProvider(provider)
 {
-    mIcon = XdgIcon::fromTheme("utilities-terminal");
+    mIcon = QIcon::fromTheme(QSL("utilities-terminal"));
 }
 
 
@@ -559,9 +561,9 @@ void CustomCommandItem::setCommand(const QString &command)
     mExec = which(expandCommand(command));
 
     if (!mExec.isEmpty())
-        mComment = QString("%1 %2").arg(mExec, command.section(' ', 1));
+        mComment = QString::fromLatin1("%1 %2").arg(mExec, command.section(QL1C(' '), 1));
     else
-        mComment = "";
+        mComment = QString();
 
 }
 
@@ -625,11 +627,11 @@ bool VirtualBoxItem::run() const
 {
     QStringList arguments;
 #ifdef VBOX_HEADLESS_ENABLED
-    arguments << "-startvm" << title();
-    return QProcess::startDetached ("VBoxHeadless" , arguments);
+    arguments << QSL("-startvm") << title();
+    return QProcess::startDetached (QSL("VBoxHeadless") , arguments);
 #else
-    arguments << "startvm" << title();
-    return QProcess::startDetached ("VBoxManage" , arguments);
+    arguments << QSL("startvm") << title();
+    return QProcess::startDetached (QSL("VBoxManage") , arguments);
 #endif
 
 }
@@ -650,7 +652,7 @@ inline QString homeDir() {
 
 ///////
 VirtualBoxProvider::VirtualBoxProvider():
-        virtualBoxConfig ( homeDir() + "/.VirtualBox/VirtualBox.xml")
+        virtualBoxConfig ( homeDir() + QSL("/.VirtualBox/VirtualBox.xml"))
 {
     fp.setFileName (virtualBoxConfig);
 
@@ -721,7 +723,7 @@ VirtualBoxProvider::VirtualBoxProvider():
 
     for (size_t n = 0; n < sizeof (kOSTypeIcons) / sizeof(kOSTypeIcons[0]); ++ n)
     {
-        osIcons.insert (kOSTypeIcons [n][0], (kOSTypeIcons [n][1]));
+        osIcons.insert (QL1S(kOSTypeIcons [n][0]), (QL1S(kOSTypeIcons [n][1])));
     }
 }
 
@@ -734,11 +736,11 @@ void VirtualBoxProvider::rebuild()
         return;
     }
 
-    QDomNodeList _dnlist = d.elementsByTagName( "MachineEntry" );
+    QDomNodeList _dnlist = d.elementsByTagName( QSL("MachineEntry") );
     for ( int i = 0; i < _dnlist.count(); i++ )
     {
         const QDomNode & node = _dnlist.at( i );
-        const QString & ref = node.toElement().attribute( "src" );
+        const QString & ref = node.toElement().attribute( QSL("src") );
         if ( ref.isEmpty() )
         {
             qDebug() << "MachineEntry with no src attribute";
@@ -754,23 +756,23 @@ void VirtualBoxProvider::rebuild()
             continue;
         }
 
-        QDomNodeList _mlist = mspec.elementsByTagName( "Machine" );
+        QDomNodeList _mlist = mspec.elementsByTagName( QSL("Machine") );
         for ( int j = 0; j < _mlist.count(); j++ )
         {
          QDomNode mnode = _mlist.at( j );
 
-         QString type = mnode.toElement().attribute( "OSType" );
+         QString type = mnode.toElement().attribute( QSL("OSType") );
          VirtualBoxItem *virtualBoxItem = new VirtualBoxItem
             (
-             mnode.toElement().attribute( "name" ) ,
-             QIcon ( osIcons.value (type , ":/vbox-icons/os_other.png") )
+             mnode.toElement().attribute( QSL("name") ) ,
+             QIcon ( osIcons.value (type , QSL(":/vbox-icons/os_other.png")) )
             );
 
-         const QDomNodeList & rdeportConfig = mnode.toElement().elementsByTagName("VRDEProperties");
+         const QDomNodeList & rdeportConfig = mnode.toElement().elementsByTagName(QSL("VRDEProperties"));
          if ( ! rdeportConfig.isEmpty() )
          {
             QDomNode portNode = rdeportConfig.at(0).firstChild();
-            virtualBoxItem->setRDEPort( portNode.toElement().attribute("value") );
+            virtualBoxItem->setRDEPort( portNode.toElement().attribute(QSL("value")) );
          }
 
          append ( virtualBoxItem );
@@ -789,17 +791,46 @@ bool VirtualBoxProvider::isOutDated() const
 
 
 #ifdef MATH_ENABLED
-#include <QtScript/QScriptEngine>
-#include <QtScript/QScriptValue>
+#include <muParser.h>
+
+class MathItem::Parser : public mu::Parser
+{
+public:
+    static void initLocale()
+    {
+        try
+        {
+            // use the system's locale instead of the "C"
+            s_locale = std::locale{""};
+        } catch (const std::runtime_error & e)
+        {
+            qWarning().noquote() << "Unable to set locale for Math, " << e.what();
+        }
+    }
+};
+static void muParserInitLocale()
+{
+    MathItem::Parser::initLocale();
+}
+Q_COREAPP_STARTUP_FUNCTION(muParserInitLocale)
 
 /************************************************
 
  ************************************************/
 MathItem::MathItem():
-        CommandProviderItem()
+        CommandProviderItem(),
+        mParser{new Parser}
 {
     mToolTip =QObject::tr("Mathematics");
-    mIcon = XdgIcon::fromTheme("accessories-calculator");
+    mIcon = QIcon::fromTheme(QSL("accessories-calculator"));
+}
+
+
+/************************************************
+
+ ************************************************/
+MathItem::~MathItem()
+{
 }
 
 
@@ -819,17 +850,42 @@ bool MathItem::compare(const QRegExp &regExp) const
 {
     QString s = regExp.pattern().trimmed();
 
-    if (s.endsWith("="))
+    bool is_math = false;
+    if (s.startsWith(QLatin1Char('=')))
     {
+        is_math = true;
+        s.remove(0, 1);
+    }
+    if (s.endsWith(QLatin1Char('=')))
+    {
+        is_math = true;
         s.chop(1);
-        QScriptEngine myEngine;
-        QScriptValue res = myEngine.evaluate(s);
-        if (res.isNumber())
+    }
+
+    if (is_math)
+    {
+        if (s != mCachedInput)
         {
-            MathItem *self=const_cast<MathItem*>(this);
-            self->mTitle = s + " = " + res.toString();
-            return true;
+            MathItem * self = const_cast<MathItem*>(this);
+            mCachedInput = s;
+            self->mTitle.clear();
+
+            //try to compute anything suitable
+            for (int attempts = 20; 0 < attempts && 0 < s.size(); s.chop(1), --attempts)
+            {
+                try
+                {
+                    mParser->SetExpr(s.toStdString());
+                    self->mTitle = s + QL1C('=') + QLocale::system().toString(mParser->Eval());
+                    break;
+                } catch (const mu::Parser::exception_type & e)
+                {
+                    //don't do anything, return false -> no result will be showed
+                }
+            }
         }
+
+        return !mTitle.isEmpty();
     }
 
     return false;
@@ -852,23 +908,25 @@ MathProvider::MathProvider()
     append(new MathItem());
 }
 
+#endif
+
 ExternalProviderItem::ExternalProviderItem()
 {
 }
 
 bool ExternalProviderItem::setData(QMap<QString,QString> & data)
 {
-    if (! (data.contains("title") && data.contains("command")))
+    if (! (data.contains(QL1S("title")) && data.contains(QL1S("dialog/monitor"))))
     {
         return false;
     }
 
-    mTitle = data["title"];
-    mComment = data["comment"];
-    mToolTip = data["tooltip"];
-    mCommand = data["command"];
-    if (data.contains("icon"))
-        mIcon = XdgIcon::fromTheme(data["icon"]);
+    mTitle = data[QLatin1String("title")];
+    mComment = data[QLatin1String("comment")];
+    mToolTip = data[QSL("tooltip")];
+    mCommand = data[QL1S("dialog/monitor")];
+    if (data.contains(QL1S("icon")))
+        mIcon = QIcon::fromTheme(data[QL1S("icon")]);
 
     return true;
 }
@@ -900,7 +958,8 @@ CommandProvider(), mName(name)
 void ExternalProvider::setSearchTerm(const QString searchTerm)
 {
     mExternalProcess->write(searchTerm.toUtf8());
-    mExternalProcess->write(QString("\n").toUtf8());
+    mExternalProcess->write(QBAL("\n"));
+//    mExternalProcess->write(QString("\n").toUtf8());
 }
 
 void ExternalProvider::newListOfMaps(QList<QMap<QString,QString> > maps)
@@ -910,8 +969,7 @@ void ExternalProvider::newListOfMaps(QList<QMap<QString,QString> > maps)
     qDeleteAll(*this);
     clear();
 
-    QMap<QString, QString> map;
-    foreach(map, maps)
+    for(auto map : qAsConst(maps))
     {
         ExternalProviderItem *item  = new ExternalProviderItem();
         if (item->setData(map))
@@ -941,4 +999,3 @@ void ExternalProvider::readFromProcess()
     }
 }
 
-#endif
